@@ -3,19 +3,17 @@
 #         Joseph Salmon <joseph.salmon@telecom-paristech.fr>
 # License: BSD 3 clause
 
-import warnings
 import numpy as np
 
 from scipy import sparse
 from numpy.linalg import norm
 from sklearn.utils import check_array
-from sklearn.exceptions import ConvergenceWarning
 from sklearn.linear_model._base import _preprocess_data
 
 from .lasso_fast import celer
 from .group_fast import celer_grp, dnorm_grp
 from .cython_utils import compute_norms_X_col, compute_Xw
-from .cython_utils import dnorm_l1 as dnorm_l1_cython
+from .cython_utils import dnorm_enet as dnorm_enet_cython
 from .multitask_fast import celer_mtl
 from .PN_logreg import newton_celer
 
@@ -24,35 +22,43 @@ LOGREG = 1
 GRPLASSO = 2
 
 
-def celer_path(X, y, pb, eps=1e-3, n_alphas=100, alphas=None,
-               coef_init=None, max_iter=20, gap_freq=10, max_epochs=50000,
+def celer_path(X, y, pb, eps=1e-3, n_alphas=100, alphas=None, l1_ratio=1.0,
+               coef_init=None, max_iter=20, max_epochs=50000,
                p0=10, verbose=0, tol=1e-6, prune=0, weights=None,
                groups=None, return_thetas=False, use_PN=False, X_offset=None,
                X_scale=None, return_n_iter=False, positive=False):
     r"""Compute optimization path with Celer as inner solver.
 
-    With `n = len(y)` and `p = len(w)` the number of samples and features,
+    With ``n = len(y)`` and ``p = len(w)`` the number of samples and features,
     the losses are:
 
-    Lasso:
+    * Lasso:
 
     .. math::
 
-        \frac{||y - X w||_2^2}{2 n} + \alpha \sum_1^p weights_j |w_j|
+        \frac{\| y - X w \||_2^2}{2 n} + \alpha \sum_{j=1}^p weights_j |w_j|
 
-    Logreg:
+    * ElasticNet:
+
+    .. math::
+
+        \frac{\| y - X w \|_2^2}{2 n} +
+        \alpha \sum_{j=1}^p weights_j (l1\_ratio |w_j| + (1-l1\_ratio) w_j^2)
+
+    * Logreg:
 
     .. math::
 
         \sum_{i=1}^n \text{log} \,(1 + e^{-y_i x_i^\top w}) + \alpha
-        \sum_1^p weights_j |w_j|
+        \sum_{j=1}^p weights_j |w_j|
 
-    GroupLasso, with `G` the number of groups and `w_{[g]}` the subvector
+    * GroupLasso, with `G` the number of groups and :math:`w_{[g]}` the subvector
     corresponding the group `g`:
 
     .. math::
 
-        \frac{||y - X w||_2^2}{2 n} + \alpha \sum_1^G weights_g ||w_{[g]}||_2
+        \frac{\| y - X w \|_2^2}{2 n} + \alpha \sum_{g=1}^G weights_g \| w_{[g]} \|_2
+
 
     Parameters
     ----------
@@ -61,31 +67,33 @@ def celer_path(X, y, pb, eps=1e-3, n_alphas=100, alphas=None,
         sparse format (CSC) to avoid unnecessary memory duplication.
 
     y : ndarray, shape (n_samples,)
-        Target values
+        Target values.
 
     pb : "lasso" | "logreg" | "grouplasso"
         Optimization problem to solve.
 
     eps : float, optional
         Length of the path. ``eps=1e-3`` means that
-        ``alpha_min = 1e-3 * alpha_max``
+        ``alpha_min = 1e-3 * alpha_max``.
 
     n_alphas : int, optional
-        Number of alphas along the regularization path
+        Number of alphas along the regularization path.
 
     alphas : ndarray, optional
         List of alphas where to compute the models.
-        If ``None`` alphas are set automatically
+        If ``None`` alphas are set automatically.
 
-    coef_init : ndarray, shape (n_features,) | None, optional, (defualt=None)
-        Initial value of coefficients. If None, np.zeros(n_features) is used.
+    l1_ratio : float, optional
+        The ElasticNet mixing parameter, with ``0 < l1_ratio <= 1``.
+        Defaults to 1.0 which corresponds to L1 penalty (Lasso).
+        ``l1_ratio = 0`` (Ridge regression) is not supported.
+
+    coef_init : ndarray, shape (n_features,) | None, optional, (default=None)
+        Initial value of coefficients. If ``None``, ``np.zeros(n_features)`` is used.
 
     max_iter : int, optional
-        The maximum number of iterations (subproblem definitions)
-
-    gap_freq : int, optional
-        Number of (block) coordinate descent epochs between each duality gap
-        computations.
+        The maximum number of iterations (definition of working set and
+        resolution of problem restricted to features in working set).
 
     max_epochs : int, optional
         Maximum number of (block) CD epochs on each subproblem.
@@ -94,7 +102,7 @@ def celer_path(X, y, pb, eps=1e-3, n_alphas=100, alphas=None,
         First working set size.
 
     verbose : bool or integer, optional
-        Amount of verbosity. 0/False is silent
+        Amount of verbosity. ``0`` or ``False`` is silent.
 
     tol : float, optional
         The tolerance for the optimization: the solver runs until the duality
@@ -106,31 +114,37 @@ def celer_path(X, y, pb, eps=1e-3, n_alphas=100, alphas=None,
 
     weights : ndarray, shape (n_features,) or (n_groups,), optional
         Feature/group weights used in the penalty. Default to array of ones.
+        Features with weights equal to ``np.inf`` are ignored.
 
     groups : int or list of ints or list of list of ints, optional
         Used for the group Lasso only. See the documentation of the
-        GroupLasso class.
+        :ref:`celer.GroupLasso` class.
 
     return_thetas : bool, optional
-        If True, dual variables along the path are returned.
+        If ``True``, dual variables along the path are returned.
 
     use_PN : bool, optional
-        If pb == "logreg", use ProxNewton solver instead of coordinate
+        If ``pb == "logreg"``, use ProxNewton solver instead of coordinate
         descent.
 
     X_offset : np.array, shape (n_features,), optional
         Used to center sparse X without breaking sparsity. Mean of each column.
-        See sklearn.linear_model.base._preprocess_data().
+        See `sklearn.linear_model.base._preprocess_data()
+        <https://github.com/scikit-learn/scikit-learn/blob
+        /213d21fe719ce5778726203893c78251b8af34fa/sklearn/linear_model/_base.py#L216>`_.
 
     X_scale : np.array, shape (n_features,), optional
         Used to scale centered sparse X without breaking sparsity. Norm of each
-        centered column. See sklearn.linear_model.base._preprocess_data().
+        centered column.
+        See `sklearn.linear_model.base._preprocess_data()
+        <https://github.com/scikit-learn/scikit-learn/blob
+        /213d21fe719ce5778726203893c78251b8af34fa/sklearn/linear_model/_base.py#L216>`_.
 
     return_n_iter : bool, optional
-        If True, number of iterations along the path are returned.
+        If ``True``, number of iterations along the path are returned.
 
     positive : bool, optional (default=False)
-        If True and pb == "lasso", forces the coefficients to be positive.
+        If ``True`` and ``pb == "lasso"``, forces the coefficients to be positive.
 
     Returns
     -------
@@ -145,11 +159,18 @@ def celer_path(X, y, pb, eps=1e-3, n_alphas=100, alphas=None,
 
     thetas : array, shape (n_alphas, n_samples)
         The dual variables along the path.
-        (Is returned only when ``return_thetas`` is set to True).
+        (``thetas`` are returned if ``return_thetas`` is set to ``True``).
     """
 
     if pb.lower() not in ("lasso", "logreg", "grouplasso"):
         raise ValueError("Unsupported problem %s" % pb)
+
+    if pb.lower() != "lasso" and l1_ratio != 1.0:
+        raise NotImplementedError(
+            "Mix of l1 and l2 penalty not supported for %s" % pb
+        )
+
+    n_groups = None  # set n_groups to None for lasso and logreg
     if pb.lower() == "lasso":
         pb = LASSO
     elif pb.lower() == "logreg":
@@ -186,27 +207,26 @@ def celer_path(X, y, pb, eps=1e-3, n_alphas=100, alphas=None,
 
     X_dense, X_data, X_indices, X_indptr = _sparse_and_dense(X)
 
-    if weights is None:
-        weights = np.ones(n_groups) if pb == GRPLASSO else np.ones(n_features)
-        weights = weights.astype(X.dtype)
+    weights = _check_weights(weights, pb, X, n_groups)
+    # to prevent ref before assignment in dnorm_enet
+    w = np.zeros(n_features, dtype=X.dtype)
 
     if alphas is None:
         if pb == LASSO:
-            alpha_max = dnorm_l1(X, y, weights, X_sparse_scaling,
-                                 positive) / n_samples
+            alpha_max = dnorm_enet(X, y, w, weights, X_sparse_scaling,
+                                   positive) / n_samples
         elif pb == LOGREG:
-            alpha_max = dnorm_l1(X, y, weights, X_sparse_scaling,
-                                 positive) / 2
+            alpha_max = dnorm_enet(X, y, w, weights, X_sparse_scaling,
+                                   positive) / 2
         elif pb == GRPLASSO:
             # TODO compute it with dscal to handle centering sparse
             alpha_max = 0
             for g in range(n_groups):
                 X_g = X[:, grp_indices[grp_ptr[g]:grp_ptr[g + 1]]]
-                alpha_max = max(alpha_max, norm(X_g.T @ y, ord=2))
+                alpha_max = max(alpha_max, norm(X_g.T @ y / weights[g], ord=2))
             alpha_max /= n_samples
 
-        alphas = alpha_max * np.geomspace(1, eps, n_alphas,
-                                          dtype=X.dtype)
+        alphas = alpha_max / l1_ratio * np.geomspace(1, eps, n_alphas, dtype=X.dtype)
     else:
         alphas = np.sort(alphas)[::-1]
 
@@ -250,6 +270,7 @@ def celer_path(X, y, pb, eps=1e-3, n_alphas=100, alphas=None,
     # do not skip alphas[0], it is not always alpha_max
     for t in range(n_alphas):
         alpha = alphas[t]
+
         if verbose:
             to_print = "##### Computing alpha %d/%d" % (t + 1, n_alphas)
             print("#" * len(to_print))
@@ -257,7 +278,8 @@ def celer_path(X, y, pb, eps=1e-3, n_alphas=100, alphas=None,
             print("#" * len(to_print))
         if t > 0:
             w = coefs[:, t - 1].copy()
-            theta = thetas[t - 1].copy()
+            # theta was feasible for alphas[t-1], make it feasible for alphas[t]
+            theta = thetas[t - 1] * (alphas[t] / alphas[t-1])
             p0 = max(len(np.where(w != 0)[0]), 1)
         else:
             if coef_init is not None:
@@ -269,7 +291,6 @@ def celer_path(X, y, pb, eps=1e-3, n_alphas=100, alphas=None,
                     is_sparse, pb, Xw, w, y, X_sparse_scaling.any(), X_dense,
                     X_data, X_indices, X_indptr, X_sparse_scaling)
             else:
-                w = np.zeros(n_features, dtype=X.dtype)
                 Xw = np.zeros(n_samples, X.dtype) if pb == LOGREG else y.copy()
 
             # different link equations and normalization scal for dual point:
@@ -278,31 +299,32 @@ def celer_path(X, y, pb, eps=1e-3, n_alphas=100, alphas=None,
                     theta = Xw.copy()
                 elif pb == LOGREG:
                     theta = y / (1 + np .exp(y * Xw)) / alpha
-                scal = dnorm_l1(X, theta, weights, X_sparse_scaling,
-                                positive)
+                scal = dnorm_enet(X, theta, w, weights, X_sparse_scaling,
+                                  positive, alpha, l1_ratio)
             elif pb == GRPLASSO:
                 theta = Xw.copy()
                 scal = dnorm_grp(
                     is_sparse, theta, grp_ptr, grp_indices, X_dense,
                     X_data, X_indices, X_indptr, X_sparse_scaling,
-                    len(grp_ptr) - 1, np.zeros(1, dtype=np.int32),
+                    weights, len(grp_ptr) - 1, np.zeros(1, dtype=np.int32),
                     X_sparse_scaling.any())
-            theta /= scal
+            theta *= alpha * l1_ratio / scal
 
         # celer modifies w, Xw, and theta in place:
-        if pb == GRPLASSO:  # TODO this if else scheme is complicated
+        if pb == GRPLASSO:
+            # TODO this if else scheme is complicated
             sol = celer_grp(
                 is_sparse, LASSO, X_dense, grp_indices, grp_ptr, X_data,
-                X_indices,
-                X_indptr, X_sparse_scaling, y, alpha, w, Xw, theta,
-                norms_X_grp, tol, max_iter, max_epochs, gap_freq, p0=p0,
+                X_indices, X_indptr, X_sparse_scaling, y, alpha, w, Xw, theta,
+                norms_X_grp, tol, weights, max_iter, max_epochs, p0=p0,
                 prune=prune, verbose=verbose)
+        # TODO handle case of enet
         elif pb == LASSO or (pb == LOGREG and not use_PN):
             sol = celer(
                 is_sparse, pb,
                 X_dense, X_data, X_indices, X_indptr, X_sparse_scaling, y,
-                alpha, w, Xw, theta, norms_X_col, weights,
-                max_iter=max_iter, gap_freq=gap_freq, max_epochs=max_epochs,
+                alpha, l1_ratio, w, Xw, theta, norms_X_col, weights,
+                max_iter=max_iter, max_epochs=max_epochs,
                 p0=p0, verbose=verbose, use_accel=1, tol=tol, prune=prune,
                 positive=positive)
         else:  # pb == LOGREG and use_PN
@@ -314,13 +336,6 @@ def celer_path(X, y, pb, eps=1e-3, n_alphas=100, alphas=None,
         if return_n_iter:
             n_iters[t] = len(sol[2])
 
-        if dual_gaps[t] > tol:
-            warnings.warn(
-                'Objective did not converge. Increasing `tol` may make the' +
-                ' solver faster without affecting the results much. \n' +
-                'Fitting data with very small alpha causes precision issues.',
-                ConvergenceWarning)
-
     results = alphas, coefs, dual_gaps
     if return_thetas:
         results += (thetas,)
@@ -330,6 +345,26 @@ def celer_path(X, y, pb, eps=1e-3, n_alphas=100, alphas=None,
     return results
 
 
+def _check_weights(weights, pb, X, n_groups):
+    """Handle weights cases."""
+    if weights is None:
+        n_weights = n_groups if pb == GRPLASSO else X.shape[1]
+        weights = np.ones(n_weights, dtype=X.dtype)
+    elif (weights <= 0).any():
+        raise ValueError("0 or negative weights are not supported.")
+    else:
+        expected_n_weights = n_groups if pb == GRPLASSO else X.shape[1]
+        feat_or_grp = "groups" if pb == GRPLASSO else "features"
+
+        if weights.shape[0] != expected_n_weights:
+            raise ValueError(
+                f"As many weights as {feat_or_grp} must be passed. "
+                f"Expected {expected_n_weights}, got {weights.shape[0]}."
+            )
+
+    return weights
+
+
 def _sparse_and_dense(X):
     if sparse.issparse(X):
         X_dense = np.empty([1, 1], order='F', dtype=X.data.dtype)
@@ -344,13 +379,15 @@ def _sparse_and_dense(X):
     return X_dense, X_data, X_indices, X_indptr
 
 
-def dnorm_l1(X, theta, weights, X_sparse_scaling, positive):
+def dnorm_enet(X, theta, w, weights, X_sparse_scaling,
+               positive, alpha=1.0, l1_ratio=1.0):
     """Theta should be centered."""
     X_dense, X_data, X_indices, X_indptr = _sparse_and_dense(X)
     skip = np.zeros(X.shape[1], dtype=np.int32)
-    scal = dnorm_l1_cython(
-        sparse.issparse(X), theta, X_dense, X_data, X_indices, X_indptr,
-        skip, X_sparse_scaling, weights, X_sparse_scaling.any(), positive)
+    scal = dnorm_enet_cython(
+        sparse.issparse(X), theta, w, X_dense, X_data, X_indices, X_indptr,
+        skip, X_sparse_scaling, weights, X_sparse_scaling.any(), positive,
+        alpha, l1_ratio)
     return scal
 
 
@@ -392,10 +429,9 @@ def _grp_converter(groups, n_features):
 
 
 def mtl_path(
-        X, Y, eps=1e-2, n_alphas=100, alphas=None, max_iter=100, gap_freq=10,
-        max_epochs=50000, p0=10, verbose=0, tol=1e-6,
-        prune=True, use_accel=True, return_thetas=False, K=6,
-        coef_init=None):
+        X, Y, eps=1e-2, n_alphas=100, alphas=None, max_iter=100,
+        max_epochs=50_000, p0=10, verbose=0, tol=1e-6,
+        prune=True, return_thetas=False, coef_init=None):
     X = check_array(X, "csc", dtype=[
         np.float64, np.float32], order="F", copy=False)
     Y = check_array(Y, "csc", dtype=[
@@ -411,11 +447,9 @@ def mtl_path(
 
     n_alphas = len(alphas)
 
-    if coef_init is None:
-        coefs = np.zeros((n_features, n_tasks, n_alphas), order="F",
-                         dtype=X.dtype)
-    else:
-        coefs = np.swapaxes(coef_init, 0, 1).copy('F')
+    coefs = np.zeros((n_features, n_tasks, n_alphas), order="F",
+                     dtype=X.dtype)
+
     thetas = np.zeros((n_alphas, n_samples, n_tasks), dtype=X.dtype)
     gaps = np.zeros(n_alphas)
 
@@ -437,14 +471,18 @@ def mtl_path(
             W = coefs[:, :, t - 1].copy()
             p_t = max(len(np.where(W[:, 0] != 0)[0]), p0)
         else:
-            W = coefs[:, :, t].copy()
-            p_t = 10
+            if coef_init is not None:
+                W = coef_init.T
+                R = np.asfortranarray(Y - X @ W)
+                p_t = max(len(np.where(W[:, 0] != 0)[0]), p0)
+            else:
+                W = np.zeros((n_features, n_tasks), dtype=X.dtype)
+                p_t = 10
 
         sol = celer_mtl(
             X, Y, alpha, W, R, theta, norms_X_col, p0=p_t, tol=tol,
             prune=prune, max_iter=max_iter, max_epochs=max_epochs,
-            verbose=verbose, use_accel=use_accel, gap_freq=gap_freq,
-            K=K)
+            verbose=verbose, use_accel=True)
 
         coefs[:, :, t], thetas[t], gaps[t] = sol[0], sol[1], sol[2]
 
