@@ -1,7 +1,9 @@
 import pytest
 import itertools
+
 import numpy as np
 from numpy.linalg import norm
+from numpy.testing import assert_allclose, assert_array_less, assert_array_equal
 
 from sklearn.utils.estimator_checks import check_estimator
 from sklearn.linear_model import MultiTaskLassoCV as sklearn_MultiTaskLassoCV
@@ -15,9 +17,9 @@ from celer.group_fast import dnorm_grp
 from celer.utils.testing import build_dataset
 
 
-@pytest.mark.parametrize("sparse_X, fit_intercept, normalize",
-                         itertools.product([0, 1], [0, 1], [0, 1]))
-def test_GroupLasso_Lasso_equivalence(sparse_X, fit_intercept, normalize):
+@pytest.mark.parametrize("sparse_X, fit_intercept",
+                         itertools.product([0, 1], [0, 1]))
+def test_GroupLasso_Lasso_equivalence(sparse_X, fit_intercept):
     """Check that GroupLasso with groups of size 1 gives Lasso."""
     n_features = 1000
     X, y = build_dataset(
@@ -25,12 +27,11 @@ def test_GroupLasso_Lasso_equivalence(sparse_X, fit_intercept, normalize):
     alpha_max = norm(X.T @ y, ord=np.inf) / len(y)
     alpha = alpha_max / 10
     clf = Lasso(alpha, tol=1e-12, fit_intercept=fit_intercept,
-                normalize=normalize, verbose=0)
+                verbose=0)
     clf.fit(X, y)
     # take groups of size 1:
     clf1 = GroupLasso(alpha=alpha, groups=1, tol=1e-12,
-                      fit_intercept=fit_intercept, normalize=normalize,
-                      verbose=0)
+                      fit_intercept=fit_intercept, verbose=0)
     clf1.fit(X, y)
 
     np.testing.assert_allclose(clf1.coef_, clf.coef_, atol=1e-6)
@@ -58,19 +59,21 @@ def test_GroupLasso_MultitaskLasso_equivalence():
     X_data = np.empty([1], dtype=X.dtype)
     X_indices = np.empty([1], dtype=np.int32)
     X_indptr = np.empty([1], dtype=np.int32)
+    weights = np.ones(len(grp_ptr) - 1)
+
     other = dnorm_grp(
         False, y, grp_ptr, grp_indices, X, X_data,
-        X_indices, X_indptr, X_data, len(grp_ptr) - 1,
+        X_indices, X_indptr, X_data, weights, len(grp_ptr) - 1,
         np.zeros(1, dtype=np.int32), False)
     np.testing.assert_allclose(alpha_max, other / len(Y_))
 
     alpha = alpha_max / 10
-    clf = MultiTaskLasso(alpha, fit_intercept=False, tol=1e-8, verbose=2)
+    clf = MultiTaskLasso(alpha, fit_intercept=False, tol=1e-8, verbose=0)
     clf.fit(X_, Y_)
 
     groups = [grp.tolist() for grp in grp_indices.reshape(50, 3)]
     clf1 = GroupLasso(alpha=alpha / 3, groups=groups,
-                      fit_intercept=False, tol=1e-8, verbose=2)
+                      fit_intercept=False, tol=1e-8, verbose=0)
     clf1.fit(X, y)
 
     np.testing.assert_allclose(clf1.coef_, clf.coef_.reshape(-1), atol=1e-4)
@@ -94,10 +97,10 @@ def test_convert_groups():
 
 def test_mtl_path():
     X, Y = build_dataset(n_targets=3)
-    tol = 1e-10
+    tol = 1e-12
     params = dict(eps=0.01, tol=tol, n_alphas=10)
     alphas, coefs, gaps = mtl_path(X, Y, **params)
-    np.testing.assert_array_less(gaps, tol)
+    np.testing.assert_array_less(gaps, tol * norm(Y) ** 2 / len(Y))
 
     sk_alphas, sk_coefs, sk_gaps = lasso_path(X, Y, **params, max_iter=10000)
     np.testing.assert_array_less(sk_gaps, tol * np.linalg.norm(Y, 'fro')**2)
@@ -109,8 +112,8 @@ def test_MultiTaskLassoCV():
     """Test that our MultitaskLassoCV behaves like sklearn's."""
     X, y = build_dataset(n_samples=30, n_features=50, n_targets=3)
 
-    params = dict(eps=1e-2, n_alphas=10, tol=1e-10, cv=2, n_jobs=1,
-                  fit_intercept=False, verbose=2)
+    params = dict(eps=1e-2, n_alphas=10, tol=1e-12, cv=2, n_jobs=1,
+                  fit_intercept=False, verbose=0)
 
     clf = MultiTaskLassoCV(**params)
     clf.fit(X, y)
@@ -128,8 +131,9 @@ def test_MultiTaskLassoCV():
 
     # check_estimator tests float32 so using tol < 1e-7 causes precision
     # issues
-    clf.tol = 1e-5
-    check_estimator(clf)
+    # we don't support sample_weights for MTL
+    # clf.tol = 1e-5
+    # check_estimator(clf)
 
 
 @pytest.mark.parametrize("fit_intercept", [True, False])
@@ -139,8 +143,7 @@ def test_MultiTaskLasso(fit_intercept):
     alpha_max = np.max(norm(X.T.dot(Y), axis=1)) / X.shape[0]
 
     alpha = alpha_max / 2.
-    params = dict(alpha=alpha, fit_intercept=fit_intercept, tol=1e-10,
-                  normalize=True)
+    params = dict(alpha=alpha, fit_intercept=fit_intercept, tol=1e-10)
     clf = MultiTaskLasso(**params)
     clf.verbose = 2
     clf.fit(X, Y)
@@ -151,8 +154,9 @@ def test_MultiTaskLasso(fit_intercept):
     if fit_intercept:
         np.testing.assert_allclose(clf.intercept_, clf2.intercept_)
 
-    clf.tol = 1e-7
-    check_estimator(clf)
+    # we don't support sample_weights for MTL
+    # clf.tol = 1e-7
+    # check_estimator(clf)
 
 
 @pytest.mark.parametrize("sparse_X", [True, False])
@@ -164,7 +168,7 @@ def test_group_lasso_path(sparse_X):
     alphas, coefs, gaps = celer_path(
         X, y, "grouplasso", groups=5, eps=1e-2, n_alphas=10, tol=1e-8)
     tol = 1e-8
-    np.testing.assert_array_less(gaps, tol)
+    np.testing.assert_array_less(gaps, tol * norm(y) ** 2 / len(y))
 
 
 @pytest.mark.parametrize("sparse_X", [True, False])
@@ -176,7 +180,7 @@ def test_GroupLasso(sparse_X):
     tol = 1e-8
     clf = GroupLasso(alpha=0.8, groups=10, tol=tol)
     clf.fit(X, y)
-    np.testing.assert_array_less(clf.dual_gap_, tol)
+    np.testing.assert_array_less(clf.dual_gap_, tol * norm(y) ** 2 / len(y))
 
     clf.tol = 1e-6
     clf.groups = 1  # unsatisfying but sklearn will fit out of 5 features
@@ -192,12 +196,75 @@ def test_GroupLassoCV(sparse_X):
     tol = 1e-8
     clf = GroupLassoCV(groups=10, tol=tol)
     clf.fit(X, y)
-    np.testing.assert_array_less(clf.dual_gap_, tol)
+    np.testing.assert_array_less(clf.dual_gap_, tol * norm(y) ** 2 / len(y))
 
     clf.tol = 1e-6
     clf.groups = 1  # unsatisfying but sklearn will fit with 5 features
     check_estimator(clf)
 
 
+def test_weights_group_lasso():
+    n_samples, n_features = 30, 50
+    X, y = build_dataset(n_samples, n_features, sparse_X=True)
+
+    groups = 5
+    n_groups = n_features // groups
+    np.random.seed(0)
+    weights = np.abs(np.random.randn(n_groups))
+
+    tol = 1e-14
+    params = {'n_alphas': 10, 'tol': tol, 'verbose': 1}
+    augmented_weights = np.repeat(weights, groups)
+
+    alphas1, coefs1, gaps1 = celer_path(
+        X, y, "grouplasso", groups=groups, weights=weights,
+        eps=1e-2, **params)
+    alphas2, coefs2, gaps2 = celer_path(
+        X.multiply(1 / augmented_weights[None, :]), y, "grouplasso",
+        groups=groups, eps=1e-2, **params)
+
+    assert_allclose(alphas1, alphas2)
+    assert_allclose(
+        coefs1, coefs2 / augmented_weights[:, None], rtol=1e-3)
+    assert_array_less(gaps1, tol * norm(y) ** 2 / len(y))
+    assert_array_less(gaps2, tol * norm(y) ** 2 / len(y))
+
+
+def test_check_weights():
+    X, y = build_dataset(30, 42)
+    weights = np.ones(X.shape[1] // 7)
+    weights[0] = 0
+    clf = GroupLasso(weights=weights, groups=7)  # groups of size 7
+    # weights must be > 0
+    np.testing.assert_raises(ValueError, clf.fit, X=X, y=y)
+    # len(weights) must be equal to number of groups (6 here)
+    clf.weights = np.ones(8)
+    np.testing.assert_raises(ValueError, clf.fit, X=X, y=y)
+
+
+def test_infinite_weights_group():
+    n_samples, n_features = 50, 100
+    X, y = build_dataset(n_samples, n_features)
+
+    np.random.seed(1)
+    group_size = 5
+    weights = np.abs(np.random.randn(n_features // group_size))
+    n_inf = 3
+    inf_indices = np.random.choice(
+        n_features // group_size, size=n_inf, replace=False)
+    weights[inf_indices] = np.inf
+    alpha_max = np.max(
+        norm((X.T @ y).reshape(-1, group_size), 2, axis=1)
+    ) / n_samples
+
+    clf = GroupLasso(
+        alpha=alpha_max / 100., weights=weights, groups=group_size, tol=1e-8
+    ).fit(X, y)
+
+    assert_array_less(clf.dual_gap_, clf.tol * norm(y) ** 2 / 2)
+    assert_array_equal(
+        norm(clf.coef_.reshape(-1, group_size), axis=1)[inf_indices], 0)
+
+
 if __name__ == "__main__":
     pass
